@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Account.h"
@@ -17,6 +20,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "text/Format.h"
 
 #include <algorithm>
+#include <numeric>
 #include <sstream>
 
 using namespace std;
@@ -35,19 +39,27 @@ void Account::Load(const DataNode &node, bool clearFirst)
 	if(clearFirst)
 	{
 		credits = 0;
-		salariesOwed = 0;
+		crewSalariesOwed = 0;
 		maintenanceDue = 0;
 		creditScore = 400;
 		mortgages.clear();
 		history.clear();
 	}
-	
+
 	for(const DataNode &child : node)
 	{
 		if(child.Token(0) == "credits" && child.Size() >= 2)
 			credits = child.Value(1);
+		else if(child.Token(0) == "salaries income")
+			for(const DataNode &grand : child)
+			{
+				if(grand.Size() < 2)
+					grand.PrintTrace("Skipping incomplete salary income:");
+				else
+					salariesIncome[grand.Token(0)] = grand.Value(1);
+			}
 		else if(child.Token(0) == "salaries" && child.Size() >= 2)
-			salariesOwed = child.Value(1);
+			crewSalariesOwed = child.Value(1);
 		else if(child.Token(0) == "maintenance" && child.Size() >= 2)
 			maintenanceDue = child.Value(1);
 		else if(child.Token(0) == "score" && child.Size() >= 2)
@@ -57,6 +69,8 @@ void Account::Load(const DataNode &node, bool clearFirst)
 		else if(child.Token(0) == "history")
 			for(const DataNode &grand : child)
 				history.push_back(grand.Value(0));
+		else
+			child.PrintTrace("Skipping unrecognized account item:");
 	}
 }
 
@@ -69,12 +83,22 @@ void Account::Save(DataWriter &out) const
 	out.BeginChild();
 	{
 		out.Write("credits", credits);
-		if(salariesOwed)
-			out.Write("salaries", salariesOwed);
+		if(salariesIncome.size() > 0)
+		{
+			out.Write("salaries income");
+			out.BeginChild();
+			{
+				for(const auto &income : salariesIncome)
+					out.Write(income.first, income.second);
+			}
+			out.EndChild();
+		}
+		if(crewSalariesOwed)
+			out.Write("salaries", crewSalariesOwed);
 		if(maintenanceDue)
 			out.Write("maintenance", maintenanceDue);
 		out.Write("score", creditScore);
-		
+
 		out.Write("history");
 		out.BeginChild();
 		{
@@ -82,7 +106,7 @@ void Account::Save(DataWriter &out) const
 				out.Write(worth);
 		}
 		out.EndChild();
-		
+
 		for(const Mortgage &mortgage : mortgages)
 			mortgage.Save(out);
 	}
@@ -114,10 +138,10 @@ void Account::PayExtra(int mortgage, int64_t amount)
 	if(static_cast<unsigned>(mortgage) >= mortgages.size() || amount > credits
 			|| amount > mortgages[mortgage].Principal())
 		return;
-	
+
 	mortgages[mortgage].PayExtra(amount);
 	credits -= amount;
-	
+
 	// If this payment was for the entire remaining amount in the mortgage,
 	// remove it from the list.
 	if(!mortgages[mortgage].Principal())
@@ -130,33 +154,33 @@ void Account::PayExtra(int mortgage, int64_t amount)
 string Account::Step(int64_t assets, int64_t salaries, int64_t maintenance)
 {
 	ostringstream out;
-	
+
 	// Keep track of what payments were made and whether any could not be made.
-	salariesOwed += salaries;
+	crewSalariesOwed += salaries;
 	maintenanceDue += maintenance;
 	bool missedPayment = false;
-	
+
 	// Crew salaries take highest priority.
-	int64_t salariesPaid = salariesOwed;
-	if(salariesOwed)
+	int64_t salariesPaid = crewSalariesOwed;
+	if(crewSalariesOwed)
 	{
-		if(salariesOwed > credits)
+		if(crewSalariesOwed > credits)
 		{
 			// If you can't pay the full salary amount, still pay some of it and
 			// remember how much back wages you owe to your crew.
 			salariesPaid = max<int64_t>(credits, 0);
-			salariesOwed -= salariesPaid;
+			crewSalariesOwed -= salariesPaid;
 			credits -= salariesPaid;
 			missedPayment = true;
 			out << "You could not pay all your crew salaries.";
 		}
 		else
 		{
-			credits -= salariesOwed;
-			salariesOwed = 0;
+			credits -= crewSalariesOwed;
+			crewSalariesOwed = 0;
 		}
 	}
-	
+
 	// Maintenance costs are dealt with after crew salaries given that they act similarly.
 	int64_t maintenancePaid = maintenanceDue;
 	if(maintenanceDue)
@@ -178,7 +202,7 @@ string Account::Step(int64_t assets, int64_t salaries, int64_t maintenance)
 			maintenanceDue = 0;
 		}
 	}
-	
+
 	// Unlike salaries, each mortgage payment must either be made in its entirety,
 	// or skipped completely (accruing interest and reducing your credit score).
 	int64_t mortgagesPaid = 0;
@@ -213,30 +237,25 @@ string Account::Step(int64_t assets, int64_t salaries, int64_t maintenance)
 		else
 			++it;
 	}
-	
+
 	// Keep track of your net worth over the last HISTORY days.
 	if(history.size() > HISTORY)
 		history.erase(history.begin());
-	history.push_back(credits + assets - salariesOwed - maintenanceDue);
-	
+	history.push_back(credits + assets - crewSalariesOwed - maintenanceDue);
+
 	// If you failed to pay any debt, your credit score drops. Otherwise, even
 	// if you have no debts, it increases. (Because, having no debts at all
 	// makes you at least as credit-worthy as someone who pays debts on time.)
 	creditScore = max(200, min(800, creditScore + (missedPayment ? -5 : 1)));
-	
+
 	// If you didn't make any payments, no need to continue further.
 	if(!(salariesPaid + maintenancePaid + mortgagesPaid + finesPaid))
 		return out.str();
 	else if(missedPayment)
 		out << " ";
-	
+
 	out << "You paid ";
-	
-	auto creditString = [](int64_t payment) -> string
-	{
-		return payment == 1 ? "1 credit" : Format::Credits(payment) + " credits";
-	};
-	
+
 	map<string, int64_t> typesPaid;
 	if(salariesPaid)
 		typesPaid["crew salaries"] = salariesPaid;
@@ -246,7 +265,7 @@ string Account::Step(int64_t assets, int64_t salaries, int64_t maintenance)
 		typesPaid["mortgages"] = mortgagesPaid;
 	if(finesPaid)
 		typesPaid["fines"] = finesPaid;
-	
+
 	// If you made payments of three or more types, the punctuation needs to
 	// include commas, so just handle that separately here.
 	if(typesPaid.size() >= 3)
@@ -254,42 +273,74 @@ string Account::Step(int64_t assets, int64_t salaries, int64_t maintenance)
 		auto it = typesPaid.begin();
 		for(unsigned int i = 0; i < typesPaid.size() - 1; ++i)
 		{
-			out << creditString(it->second) << " in " << it->first << ", ";
+			out << Format::CreditString(it->second) << " in " << it->first << ", ";
 			++it;
 		}
-		out << "and " << creditString(it->second) << " in " << it->first + ".";
+		out << "and " << Format::CreditString(it->second) << " in " << it->first + ".";
 	}
 	else
 	{
 		if(salariesPaid)
-			out << creditString(salariesPaid) << " in crew salaries"
+			out << Format::CreditString(salariesPaid) << " in crew salaries"
 				<< ((mortgagesPaid || finesPaid || maintenancePaid) ? " and " : ".");
 		if(maintenancePaid)
-			out << creditString(maintenancePaid) << "  in maintenance"
+			out << Format::CreditString(maintenancePaid) << "  in maintenance"
 				<< ((mortgagesPaid || finesPaid) ? " and " : ".");
 		if(mortgagesPaid)
-			out << creditString(mortgagesPaid) << " in mortgages"
+			out << Format::CreditString(mortgagesPaid) << " in mortgages"
 				<< (finesPaid ? " and " : ".");
 		if(finesPaid)
-			out << creditString(finesPaid) << " in fines.";
+			out << Format::CreditString(finesPaid) << " in fines.";
 	}
 	return out.str();
 }
 
 
 
-int64_t Account::SalariesOwed() const
+const map<string, int64_t> &Account::SalariesIncome() const
 {
-	return salariesOwed;
+	return salariesIncome;
+}
+
+
+
+int64_t Account::SalariesIncomeTotal() const
+{
+	return accumulate(
+		salariesIncome.begin(),
+		salariesIncome.end(),
+		0,
+		[](int64_t value, const std::map<string, int64_t>::value_type &salary)
+		{
+			return value + salary.second;
+		}
+	);
+}
+
+
+
+void Account::SetSalaryIncome(string name, int64_t amount)
+{
+	if(amount == 0)
+		salariesIncome.erase(name);
+	else
+		salariesIncome[name] = amount;
+}
+
+
+
+int64_t Account::CrewSalariesOwed() const
+{
+	return crewSalariesOwed;
 }
 
 
 
 void Account::PaySalaries(int64_t amount)
 {
-	amount = min(min(amount, salariesOwed), credits);
+	amount = min(min(amount, crewSalariesOwed), credits);
 	credits -= amount;
-	salariesOwed -= amount;
+	crewSalariesOwed -= amount;
 }
 
 
@@ -346,7 +397,7 @@ int64_t Account::Prequalify() const
 		payments += mortgage.Payment();
 		liabilities += mortgage.Principal();
 	}
-	
+
 	// Put a limit on new debt that the player can take out, as a fraction of
 	// their net worth, to avoid absurd mortgages being offered when the player
 	// has just captured some very lucrative ships.
@@ -380,7 +431,7 @@ int64_t Account::TotalDebt(const string &type) const
 	for(const Mortgage &mortgage : mortgages)
 		if(type.empty() || mortgage.Type() == type)
 			total += mortgage.Principal();
-	
+
 	return total;
 }
 
@@ -392,7 +443,7 @@ int64_t Account::YearlyRevenue() const
 {
 	if(history.empty() || history.back() <= history.front())
 		return 0;
-	
+
 	// Note that this intentionally under-estimates if the player has not yet
 	// played for long enough to accumulate a full income history.
 	return ((history.back() - history.front()) * 365) / HISTORY;
