@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "BoardingPanel.h"
@@ -48,7 +51,7 @@ namespace {
 		string result = to_string(integer / 10);
 		result += ".0";
 		result.back() += integer % 10;
-		
+
 		return result;
 	}
 }
@@ -62,14 +65,14 @@ BoardingPanel::BoardingPanel(PlayerInfo &player, const shared_ptr<Ship> &victim)
 {
 	// The escape key should close this panel rather than bringing up the main menu.
 	SetInterruptible(false);
-	
+
 	// Figure out how much the victim's commodities are worth in the current
 	// system and add them to the list of plunder.
 	const System &system = *player.GetSystem();
 	for(const auto &it : victim->Cargo().Commodities())
 		if(it.second)
 			plunder.emplace_back(it.first, it.second, system.Trade(it.first));
-	
+
 	// You cannot plunder hand to hand weapons, because they are kept in the
 	// crew's quarters, not mounted on the exterior of the ship. Certain other
 	// outfits are also unplunderable, like outfits expansions.
@@ -81,7 +84,7 @@ BoardingPanel::BoardingPanel(PlayerInfo &player, const shared_ptr<Ship> &victim)
 		int count = 0;
 		// Merge the outfit lists from the ship itself and its cargo bay. If an
 		// outfit exists in both locations, combine the counts.
-		bool shipIsFirst = (cit == victim->Cargo().Outfits().end() || 
+		bool shipIsFirst = (cit == victim->Cargo().Outfits().end() ||
 			(sit != victim->Outfits().end() && sit->first <= cit->first));
 		bool cargoIsFirst = (sit == victim->Outfits().end() ||
 			(cit != victim->Cargo().Outfits().end() && cit->first <= sit->first));
@@ -103,11 +106,12 @@ BoardingPanel::BoardingPanel(PlayerInfo &player, const shared_ptr<Ship> &victim)
 		if(outfit && count)
 			plunder.emplace_back(outfit, count);
 	}
-	
+
+	canCapture = victim->IsCapturable() || player.CaptureOverriden(victim);
 	// Some "ships" do not represent something the player could actually pilot.
-	if(!victim->IsCapturable())
+	if(!canCapture)
 		messages.emplace_back("This is not a ship that you can capture.");
-	
+
 	// Sort the plunder by price per ton.
 	sort(plunder.begin(), plunder.end());
 }
@@ -119,7 +123,7 @@ void BoardingPanel::Draw()
 {
 	// Draw a translucent black scrim over everything beneath this panel.
 	DrawBackdrop();
-	
+
 	// Draw the list of plunder.
 	const Color &opaque = *GameData::Colors().Get("panel background");
 	const Color &back = *GameData::Colors().Get("faint");
@@ -127,23 +131,23 @@ void BoardingPanel::Draw()
 	const Color &medium = *GameData::Colors().Get("medium");
 	const Color &bright = *GameData::Colors().Get("bright");
 	FillShader::Fill(Point(-155., -60.), Point(360., 250.), opaque);
-	
+
 	int index = (scroll - 10) / 20;
 	int y = -170 - scroll + 20 * index;
 	int endY = 60;
-	
+
 	const Font &font = FontSet::Get(14);
 	// Y offset to center the text in a 20-pixel high row.
 	double fontOff = .5 * (20 - font.Height());
 	for( ; y < endY && static_cast<unsigned>(index) < plunder.size(); y += 20, ++index)
 	{
 		const Plunder &item = plunder[index];
-		
+
 		// Check if this is the selected row.
 		bool isSelected = (index == selected);
 		if(isSelected)
 			FillShader::Fill(Point(-155., y + 10.), Point(360., 20.), back);
-		
+
 		// Color the item based on whether you have space for it.
 		const Color &color = item.CanTake(*you) ? isSelected ? bright : medium : dim;
 		Point pos(-320., y + fontOff);
@@ -151,7 +155,7 @@ void BoardingPanel::Draw()
 		font.Draw({item.Value(), {260, Alignment::RIGHT}}, pos, color);
 		font.Draw({item.Size(), {330, Alignment::RIGHT}}, pos, color);
 	}
-	
+
 	// Set which buttons are active.
 	Information info;
 	if(CanExit())
@@ -164,7 +168,7 @@ void BoardingPanel::Draw()
 		info.SetCondition("can attack");
 	if(CanAttack())
 		info.SetCondition("can defend");
-	
+
 	// This should always be true, but double check.
 	int crew = 0;
 	if(you)
@@ -178,7 +182,7 @@ void BoardingPanel::Draw()
 			Round(defenseOdds.DefenderPower(crew)));
 	}
 	int vCrew = victim ? victim->Crew() : 0;
-	if(victim && (victim->IsCapturable() || victim->IsYours()))
+	if(victim && (canCapture || victim->IsYours()))
 	{
 		info.SetString("enemy crew", to_string(vCrew));
 		info.SetString("enemy attack",
@@ -186,7 +190,7 @@ void BoardingPanel::Draw()
 		info.SetString("enemy defense",
 			Round(attackOdds.DefenderPower(vCrew)));
 	}
-	if(victim && victim->IsCapturable() && !victim->IsYours())
+	if(victim && canCapture && !victim->IsYours())
 	{
 		// If you haven't initiated capture yet, show the self destruct odds in
 		// the attack odds. It's illogical for you to have access to that info,
@@ -203,10 +207,10 @@ void BoardingPanel::Draw()
 		info.SetString("defense casualties",
 			Round(defenseOdds.DefenderCasualties(vCrew, crew)));
 	}
-	
+
 	const Interface *boarding = GameData::Interfaces().Get("boarding");
 	boarding->Draw(info, this);
-	
+
 	// Draw the status messages from hand to hand combat.
 	Point messagePos(50., 55.);
 	for(const string &message : messages)
@@ -234,7 +238,7 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 	{
 		CargoHold &cargo = you->Cargo();
 		int count = plunder[selected].Count();
-		
+
 		const Outfit *outfit = plunder[selected].GetOutfit();
 		if(outfit)
 		{
@@ -260,7 +264,7 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 		}
 		else
 			count = victim->Cargo().Transfer(plunder[selected].Name(), count, cargo);
-		
+
 		// If all of the plunder of this type was taken, remove it from the list.
 		// Otherwise, just update the count in the list item.
 		if(count == plunder[selected].Count())
@@ -271,24 +275,10 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 		else
 			plunder[selected].Take(count);
 	}
-	else if((key == SDLK_UP || key == SDLK_DOWN || key == SDLK_PAGEUP || key == SDLK_PAGEDOWN) && !isCapturing)
-	{
-		// Scrolling the list of plunder.
-		if(key == SDLK_PAGEUP || key == SDLK_PAGEDOWN)
-			Drag(0, 200 * ((key == SDLK_PAGEDOWN) - (key == SDLK_PAGEUP)));
-		else
-		{
-			if(key == SDLK_UP && selected)
-				--selected;
-			else if(key == SDLK_DOWN && selected < static_cast<int>(plunder.size() - 1))
-				++selected;
-			
-			// Scroll down at least far enough to view the current item.
-			double minimumScroll = max(0., 20. * selected - 200.);
-			double maximumScroll = 20. * selected;
-			scroll = max(minimumScroll, min(maximumScroll, scroll));
-		}
-	}
+	else if(!isCapturing &&
+			(key == SDLK_UP || key == SDLK_DOWN || key == SDLK_PAGEUP
+			|| key == SDLK_PAGEDOWN || key == SDLK_HOME || key == SDLK_END))
+		DoKeyboardNavigation(key);
 	else if(key == 'c' && CanCapture())
 	{
 		// A ship that self-destructs checks once when you board it, and again
@@ -298,7 +288,8 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 		{
 			victim->SelfDestruct();
 			GetUI()->Pop(this);
-			GetUI()->Push(new Dialog("The moment you blast through the airlock, a series of explosions rocks the enemy ship. They appear to have set off their self-destruct sequence..."));
+			GetUI()->Push(new Dialog("The moment you blast through the airlock, a series of explosions rocks the enemy ship."
+				" They appear to have set off their self-destruct sequence..."));
 			return true;
 		}
 		isCapturing = true;
@@ -309,7 +300,7 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 	{
 		int yourStartCrew = you->Crew();
 		int enemyStartCrew = victim->Crew();
-		
+
 		// Figure out what action the other ship will take. As a special case,
 		// if you board them but immediately "defend" they will let you return
 		// to your ship in peace. That is to allow the player to "cancel" if
@@ -319,7 +310,7 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 		if(isFirstCaptureAction && !youAttack)
 			enemyAttacks = false;
 		isFirstCaptureAction = false;
-		
+
 		// If neither side attacks, combat ends.
 		if(!youAttack && !enemyAttacks)
 		{
@@ -332,7 +323,7 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 				messages.push_back("You attack. ");
 			else if(enemyAttacks)
 				messages.push_back("You defend. ");
-			
+
 			// To speed things up, have multiple rounds of combat each time you
 			// click the button, if you started with a lot of crew.
 			int rounds = max(1, yourStartCrew / 5);
@@ -342,24 +333,24 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 				int enemyCrew = victim->Crew();
 				if(!yourCrew || !enemyCrew)
 					break;
-				
+
 				// Your chance of winning this round is equal to the ratio of
 				// your power to the enemy's power.
 				double yourPower = (youAttack ?
 					attackOdds.AttackerPower(yourCrew) : defenseOdds.DefenderPower(yourCrew));
 				double enemyPower = (enemyAttacks ?
 					defenseOdds.AttackerPower(enemyCrew) : attackOdds.DefenderPower(enemyCrew));
-				
+
 				double total = yourPower + enemyPower;
 				if(!total)
 					break;
-				
+
 				if(Random::Real() * total >= yourPower)
 					you->AddCrew(-1);
 				else
 					victim->AddCrew(-1);
 			}
-			
+
 			// Report how many casualties each side suffered.
 			int yourCasualties = yourStartCrew - you->Crew();
 			int enemyCasualties = enemyStartCrew - victim->Crew();
@@ -370,7 +361,7 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 				messages.back() += "You lose " + to_string(yourCasualties) + " crew.";
 			else if(enemyCasualties)
 				messages.back() += "They lose " + to_string(enemyCasualties) + " crew.";
-			
+
 			// Check if either ship has been captured.
 			if(!you->Crew())
 			{
@@ -382,8 +373,18 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 			else if(!victim->Crew())
 			{
 				messages.push_back("You have succeeded in capturing this ship.");
-				victim->GetGovernment()->Offend(ShipEvent::CAPTURE, victim->RequiredCrew());
-				victim->WasCaptured(you);
+				victim->GetGovernment()->Offend(ShipEvent::CAPTURE, victim->CrewValue());
+				int crewTransferred = victim->WasCaptured(you);
+				if(crewTransferred > 0)
+				{
+					string transferMessage = Format::Number(crewTransferred) + " crew member";
+					if(crewTransferred == 1)
+						transferMessage += " has";
+					else
+						transferMessage += "s have";
+					transferMessage += " been transferred.";
+					messages.push_back(transferMessage);
+				}
 				if(!victim->JumpsRemaining() && you->CanRefuel(*victim))
 					you->TransferFuel(victim->JumpFuelMissing(), &*victim);
 				player.AddShip(victim);
@@ -394,7 +395,7 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 						player.HandleEvent(ShipEvent(you, bay.ship, ShipEvent::CAPTURE), GetUI());
 					}
 				isCapturing = false;
-				
+
 				// Report this ship as captured in case any missions care.
 				ShipEvent event(you, victim, ShipEvent::CAPTURE);
 				player.HandleEvent(event, GetUI());
@@ -403,11 +404,11 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command,
 	}
 	else if(command.Has(Command::INFO))
 		GetUI()->Push(new ShipInfoPanel(player));
-	
+
 	// Trim the list of status messages.
 	while(messages.size() > 5)
 		messages.erase(messages.begin());
-	
+
 	return true;
 }
 
@@ -424,7 +425,7 @@ bool BoardingPanel::Click(int x, int y, int clicks)
 			selected = index;
 		return true;
 	}
-	
+
 	return true;
 }
 
@@ -437,7 +438,7 @@ bool BoardingPanel::Drag(double dx, double dy)
 	// and the bottom, so:
 	double maximumScroll = max(0., 20. * plunder.size() - 220.);
 	scroll = max(0., min(maximumScroll, scroll - dy));
-	
+
 	return true;
 }
 
@@ -471,7 +472,7 @@ bool BoardingPanel::CanTake() const
 		return false;
 	if(static_cast<unsigned>(selected) >= plunder.size())
 		return false;
-	
+
 	return plunder[selected].CanTake(*you);
 }
 
@@ -483,15 +484,15 @@ bool BoardingPanel::CanCapture() const
 	// You can't click the "capture" button if you're already in combat mode.
 	if(isCapturing || playerDied)
 		return false;
-	
+
 	// If your ship or the other ship has been captured:
 	if(!you->IsYours())
 		return false;
 	if(victim->IsYours())
 		return false;
-	if(!victim->IsCapturable())
+	if(!canCapture)
 		return false;
-	
+
 	return (!victim->RequiredCrew() || you->Crew() > 1);
 }
 
@@ -504,6 +505,33 @@ bool BoardingPanel::CanAttack() const
 }
 
 
+// Handle the keyboard scrolling and selection in the panel list.
+void BoardingPanel::DoKeyboardNavigation(const SDL_Keycode key)
+{
+	// Scrolling the list of plunder.
+	if(key == SDLK_PAGEUP || key == SDLK_PAGEDOWN)
+		// Keep one of the previous items onscreen while paging through.
+		selected += 10 * ((key == SDLK_PAGEDOWN) - (key == SDLK_PAGEUP));
+	else if(key == SDLK_HOME)
+		selected = 0;
+	else if(key == SDLK_END)
+		selected = static_cast<int>(plunder.size() - 1);
+	else
+	{
+		if(key == SDLK_UP)
+			--selected;
+		else if(key == SDLK_DOWN)
+			++selected;
+	}
+	selected = max(0, min(static_cast<int>(plunder.size() - 1), selected));
+
+	// Scroll down at least far enough to view the current item.
+	double minimumScroll = max(0., 20. * selected - 200.);
+	double maximumScroll = 20. * selected;
+	scroll = max(minimumScroll, min(maximumScroll, scroll));
+}
+
+
 
 // Functions for BoardingPanel::Plunder:
 
@@ -518,7 +546,7 @@ BoardingPanel::Plunder::Plunder(const string &commodity, int count, int unitValu
 
 // Constructor (outfit installed in the victim ship or transported as cargo).
 BoardingPanel::Plunder::Plunder(const Outfit *outfit, int count)
-	: name(outfit->Name()), outfit(outfit), count(count),
+	: name(outfit->DisplayName()), outfit(outfit), count(count),
 	unitValue(outfit->Cost() * (outfit->Get("installable") < 0. ? 1 : Depreciation::Full()))
 {
 	UpdateStrings();
@@ -560,8 +588,7 @@ const string &BoardingPanel::Plunder::Name() const
 
 
 
-// Get the mass, in the format "<count> x <unit mass>". If this is a
-// commodity, no unit mass is given (because it is 1). If the count is
+// Get the mass, in the format "<count> x <unit mass>". If the count is
 // 1, only the unit mass is reported.
 const string &BoardingPanel::Plunder::Size() const
 {
@@ -594,14 +621,14 @@ bool BoardingPanel::Plunder::CanTake(const Ship &ship) const
 	double mass = UnitMass();
 	if(ship.Cargo().Free() >= mass)
 		return true;
-	
+
 	// Otherwise, check if it is ammo for any of your weapons. If so, check if
 	// you can install it as an outfit.
 	if(outfit)
 		for(const auto &it : ship.Outfits())
 			if(it.first != outfit && it.first->Ammo() == outfit && ship.Attributes().CanAdd(*outfit))
 				return true;
-	
+
 	return false;
 }
 
@@ -620,13 +647,11 @@ void BoardingPanel::Plunder::Take(int count)
 void BoardingPanel::Plunder::UpdateStrings()
 {
 	double mass = UnitMass();
-	if(!outfit)
-		size = to_string(count);
-	else if(count == 1)
+	if(count == 1)
 		size = Format::Number(mass);
 	else
 		size = to_string(count) + " x " + Format::Number(mass);
-	
+
 	value = Format::Credits(unitValue * count);
 }
 
