@@ -1,4 +1,4 @@
-/* DataWriter.h
+/* DataWriter.cpp
 Copyright (c) 2014 by Michael Zahniser
 
 Endless Sky is free software: you can redistribute it and/or modify it under the
@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "DataWriter.h"
@@ -27,7 +30,16 @@ const string DataWriter::space = " ";
 
 // Constructor, specifying the file to save.
 DataWriter::DataWriter(const string &path)
-	: path(path), before(&indent)
+	: DataWriter()
+{
+	this->path = path;
+}
+
+
+
+// Constructor for a DataWriter that will not save its contents automatically
+DataWriter::DataWriter()
+	: before(&indent)
 {
 	out.precision(8);
 }
@@ -37,7 +49,16 @@ DataWriter::DataWriter(const string &path)
 // Destructor, which saves the file all in one block.
 DataWriter::~DataWriter()
 {
-	Files::Write(path, out.str());
+	if(!path.empty())
+		SaveToPath(path);
+}
+
+
+
+// Save the contents to a file.
+void DataWriter::SaveToPath(const std::string &filepath)
+{
+	Files::Write(filepath, out.str());
 }
 
 
@@ -49,7 +70,7 @@ void DataWriter::Write(const DataNode &node)
 	for(int i = 0; i < node.Size(); ++i)
 		WriteToken(node.Token(i).c_str());
 	Write();
-	
+
 	// If this node has any children, call this function recursively on them.
 	if(node.HasChildren())
 	{
@@ -100,27 +121,7 @@ void DataWriter::WriteComment(const string &str)
 // Write a token, given as a character string.
 void DataWriter::WriteToken(const char *a)
 {
-	// Figure out what kind of quotation marks need to be used for this string.
-	bool hasSpace = !*a;
-	bool hasQuote = false;
-	for(const char *it = a; *it; ++it)
-	{
-		hasSpace |= (*it <= ' ');
-		hasQuote |= (*it == '"');
-	}
-	
-	// Write the token, enclosed in quotes if necessary.
-	out << *before;
-	if(hasSpace && hasQuote)
-		out << '`' << a << '`';
-	else if(hasSpace)
-		out << '"' << a << '"';
-	else
-		out << a;
-	
-	// The next token written will not be the first one on this line, so it only
-	// needs to have a single space before it.
-	before = &space;
+	WriteToken(string(a));
 }
 
 
@@ -128,5 +129,19 @@ void DataWriter::WriteToken(const char *a)
 // Write a token, given as a string object.
 void DataWriter::WriteToken(const string &a)
 {
-	WriteToken(a.c_str());
+	// Figure out what kind of quotation marks need to be used for this string.
+	bool hasSpace = any_of(a.begin(), a.end(), [](char c) { return isspace(c); });
+	bool hasQuote = any_of(a.begin(), a.end(), [](char c) { return (c == '"'); });
+	// Write the token, enclosed in quotes if necessary.
+	out << *before;
+	if(hasQuote)
+		out << '`' << a << '`';
+	else if(hasSpace)
+		out << '"' << a << '"';
+	else
+		out << a;
+
+	// The next token written will not be the first one on this line, so it only
+	// needs to have a single space before it.
+	before = &space;
 }
