@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef ESCORT_DISPLAY_H_
@@ -20,6 +23,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include <string>
 #include <vector>
 
+class Rectangle;
 class Ship;
 class Sprite;
 
@@ -31,28 +35,28 @@ class EscortDisplay {
 public:
 	void Clear();
 	void Add(const Ship &ship, bool isHere, bool fleetIsJumping, bool isSelected);
-	
-	// The display starts in the lower left corner of the screen and takes up
-	// all but the top 450 pixels of the screen.
-	void Draw() const;
-	
+
+	// Draw as many escort icons as will fit in the given bounding box.
+	void Draw(const Rectangle &bounds) const;
+
 	// Check if the given point is a click on an escort icon. If so, return the
 	// stack of ships represented by the icon. Otherwise, return an empty stack.
 	const std::vector<const Ship *> &Click(const Point &point) const;
-	
-	
+
+
 private:
 	class Icon {
 	public:
 		Icon(const Ship &ship, bool isHere, bool fleetIsJumping, bool isSelected);
-		
+
 		// Sorting operator.
 		bool operator<(const Icon &other) const;
-		
+
 		int Height() const;
 		void Merge(const Icon &other);
-		
+
 		const Sprite *sprite;
+		bool isDisabled;
 		bool isHere;
 		bool isHostile;
 		bool notReadyToJump;
@@ -64,12 +68,12 @@ private:
 		std::vector<double> high;
 		std::vector<const Ship *> ships;
 	};
-	
-	
+
+
 private:
-	void MergeStacks() const;
-	
-	
+	void MergeStacks(int maxHeight) const;
+
+
 private:
 	mutable std::list<Icon> icons;
 	mutable std::vector<std::vector<const Ship *>> stacks;
