@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Information.h"
@@ -18,12 +21,11 @@ using namespace std;
 
 
 
-void Information::SetSprite(const string &name, const Sprite *sprite, const Point &unit, int frame)
+void Information::SetSprite(const string &name, const Sprite *sprite, const Point &unit, float frame)
 {
 	sprites[name] = sprite;
 	spriteUnits[name] = unit;
-	if(frame)
-		spriteFrames[name] = frame;
+	spriteFrames[name] = frame;
 }
 
 
@@ -31,7 +33,7 @@ void Information::SetSprite(const string &name, const Sprite *sprite, const Poin
 const Sprite *Information::GetSprite(const string &name) const
 {
 	static const Sprite empty;
-	
+
 	auto it = sprites.find(name);
 	return (it == sprites.end()) ? &empty : it->second;
 }
@@ -41,17 +43,17 @@ const Sprite *Information::GetSprite(const string &name) const
 const Point &Information::GetSpriteUnit(const string &name) const
 {
 	static const Point up(0., -1.);
-	
+
 	auto it = spriteUnits.find(name);
 	return (it == spriteUnits.end()) ? up : it->second;
 }
 
 
 
-int Information::GetSpriteFrame(const string &name) const
+float Information::GetSpriteFrame(const string &name) const
 {
 	auto it = spriteFrames.find(name);
-	return (it == spriteFrames.end()) ? 0 : it->second;
+	return (it == spriteFrames.end()) ? 0.f : it->second;
 }
 
 
@@ -66,7 +68,7 @@ void Information::SetString(const string &name, const string &value)
 const string &Information::GetString(const string &name) const
 {
 	static const string empty;
-	
+
 	auto it = strings.find(name);
 	return (it == strings.end()) ? empty : it->second;
 }
@@ -76,7 +78,7 @@ const string &Information::GetString(const string &name) const
 void Information::SetBar(const string &name, double value, double segments)
 {
 	bars[name] = value;
-	barSegments[name] = static_cast<double>(segments);
+	barSegments[name] = segments;
 }
 
 
@@ -84,8 +86,8 @@ void Information::SetBar(const string &name, double value, double segments)
 double Information::BarValue(const string &name) const
 {
 	auto it = bars.find(name);
-	
-	return (it == bars.end()) ? 1. : it->second;
+
+	return (it == bars.end()) ? 0. : it->second;
 }
 
 
@@ -93,12 +95,12 @@ double Information::BarValue(const string &name) const
 double Information::BarSegments(const string &name) const
 {
 	auto it = barSegments.find(name);
-	
+
 	return (it == barSegments.end()) ? 1. : it->second;
 }
 
 
-	
+
 void Information::SetCondition(const string &condition)
 {
 	conditions.insert(condition);
@@ -110,15 +112,15 @@ bool Information::HasCondition(const string &condition) const
 {
 	if(condition.empty())
 		return true;
-	
+
 	if(condition.front() == '!')
 		return !HasCondition(condition.substr(1));
-	
+
 	return conditions.count(condition);
 }
 
 
-	
+
 void Information::SetOutlineColor(const Color &color)
 {
 	outlineColor = color;
