@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "LoadPanel.h"
@@ -38,7 +41,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "text/truncate.hpp"
 #include "UI.h"
 
-#include "gl_header.h"
+#include "opengl.h"
 
 #include <algorithm>
 #include <stdexcept>
@@ -51,16 +54,19 @@ namespace {
 	{
 		static const size_t BUF_SIZE = 24;
 		char buf[BUF_SIZE];
-		
-		const tm *date = localtime(&timestamp);
+
 #ifdef _WIN32
+		tm date;
+		localtime_s(&date, &timestamp);
 		static const char *FORMAT = "%#I:%M %p on %#d %b %Y";
+		return string(buf, strftime(buf, BUF_SIZE, FORMAT, &date));
 #else
+		const tm *date = localtime(&timestamp);
 		static const char *FORMAT = "%-I:%M %p on %-d %b %Y";
-#endif
 		return string(buf, strftime(buf, BUF_SIZE, FORMAT, date));
+#endif
 	}
-	
+
 	// Extract the date from this pilot's most recent save.
 	string FileDate(const string &filename)
 	{
@@ -84,7 +90,7 @@ namespace {
 			}
 		return date;
 	}
-	
+
 	// Only show tooltips if the mouse has hovered in one place for this amount
 	// of time.
 	const int HOVER_TIME = 60;
@@ -113,7 +119,7 @@ void LoadPanel::Draw()
 	glClear(GL_COLOR_BUFFER_BIT);
 	GameData::Background().Draw(Point(), Point());
 	const Font &font = FontSet::Get(14);
-	
+
 	Information info;
 	if(loadedInfo.IsLoaded())
 	{
@@ -133,7 +139,7 @@ void LoadPanel::Draw()
 	}
 	else
 		info.SetString("pilot", "No Pilot Loaded");
-	
+
 	if(!selectedPilot.empty())
 		info.SetCondition("pilot selected");
 	if(!player.IsDead() && player.IsLoaded() && !selectedPilot.empty())
@@ -142,11 +148,11 @@ void LoadPanel::Draw()
 		info.SetCondition("snapshot selected");
 	if(loadedInfo.IsLoaded())
 		info.SetCondition("pilot loaded");
-	
+
 	GameData::Interfaces().Get("menu background")->Draw(info, this);
 	GameData::Interfaces().Get("load menu")->Draw(info, this);
 	GameData::Interfaces().Get("menu player info")->Draw(info, this);
-	
+
 	// The list has space for 14 entries. Alpha should be 100% for Y = -157 to
 	// 103, and fade to 0 at 10 pixels beyond that.
 	Point point(-470., -157. - sideScroll);
@@ -155,19 +161,19 @@ void LoadPanel::Draw()
 	{
 		Rectangle zone(point + Point(110., centerY), Point(230., 20.));
 		bool isHighlighted = (it.first == selectedPilot || (hasHover && zone.Contains(hoverPoint)));
-		
+
 		double alpha = min(1., max(0., min(.1 * (113. - point.Y()), .1 * (point.Y() - -167.))));
 		if(it.first == selectedPilot)
 			FillShader::Fill(zone.Center(), zone.Dimensions(), Color(.1 * alpha, 0.));
 		font.Draw({it.first, {220, Truncate::BACK}}, point, Color((isHighlighted ? .7 : .5) * alpha, 0.));
 		point += Point(0., 20.);
 	}
-	
+
 	// The hover count "decays" over time if not hovering over a saved game.
 	if(hoverCount)
 		--hoverCount;
 	string hoverText;
-	
+
 	if(!selectedPilot.empty() && files.count(selectedPilot))
 	{
 		point = Point(-110., -157. - centerScroll);
@@ -184,7 +190,7 @@ void LoadPanel::Draw()
 				if(hoverCount == HOVER_TIME)
 					hoverText = TimestampString(it.second);
 			}
-			
+
 			if(file == selectedFile)
 				FillShader::Fill(zone.Center(), zone.Dimensions(), Color(.1 * alpha, 0.));
 			size_t pos = file.find('~') + 1;
@@ -196,7 +202,7 @@ void LoadPanel::Draw()
 	if(!hoverText.empty())
 	{
 		Point boxSize(font.Width(hoverText) + 20., 30.);
-		
+
 		FillShader::Fill(hoverPoint + .5 * boxSize, boxSize, *GameData::Colors().Get("tooltip background"));
 		font.Draw(hoverText, hoverPoint + Point(10., 10.), *GameData::Colors().Get("medium"));
 	}
@@ -213,18 +219,20 @@ bool LoadPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 		// StartConditionsPanel also handles the case where there's no scenarios.
 		GetUI()->Push(new StartConditionsPanel(player, gamePanels, GameData::StartOptions(), this));
 	}
-	else if(key == 'D' && !selectedPilot.empty())
+	else if(key == 'd' && !selectedPilot.empty())
 	{
 		GetUI()->Push(new Dialog(this, &LoadPanel::DeletePilot,
-			"Are you sure you want to delete the selected pilot, \"" + selectedPilot
-				+ "\", and all their saved games?\n\n(This will permanently delete the pilot data.)"));
+			"Are you sure you want to delete the selected pilot, \"" + loadedInfo.Name()
+				+ "\", and all their saved games?\n\n(This will permanently delete the pilot data.)\n"
+				+ "Confirm the name of the pilot you want to delete.",
+				[this](const string &pilot) { return pilot == loadedInfo.Name(); }));
 	}
 	else if(key == 'a' && !player.IsDead() && player.IsLoaded())
 	{
 		auto it = files.find(selectedPilot);
 		if(it == files.end() || it->second.empty() || it->second.front().first.size() < 4)
 			return false;
-		
+
 		nameToConfirm.clear();
 		string lastSave = Files::Saves() + it->second.front().first;
 		GetUI()->Push(new Dialog(this, &LoadPanel::SnapshotCallback,
@@ -262,7 +270,7 @@ bool LoadPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 			for( ; it != files.end(); ++it)
 				if(it->first == selectedPilot)
 					break;
-			
+
 			if(key == SDLK_DOWN)
 			{
 				++it;
@@ -284,7 +292,7 @@ bool LoadPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 			for( ; it != pit->second.end(); ++it)
 				if(it->first == selectedFile)
 					break;
-			
+
 			if(key == SDLK_DOWN)
 			{
 				++it;
@@ -307,7 +315,7 @@ bool LoadPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 		sideHasFocus = false;
 	else
 		return false;
-	
+
 	return true;
 }
 
@@ -317,11 +325,11 @@ bool LoadPanel::Click(int x, int y, int clicks)
 {
 	// When the user clicks, clear the hovered state.
 	hasHover = false;
-	
+
 	// The first row of each panel is y = -160 to -140.
 	if(y < -160 || y >= (-160 + 14 * 20))
 		return false;
-	
+
 	if(x >= -470 && x < -250)
 	{
 		int selected = (y + sideScroll - -160) / 20;
@@ -344,18 +352,19 @@ bool LoadPanel::Click(int x, int y, int clicks)
 		for(const auto &it : filesIt->second)
 			if(i++ == selected)
 			{
+				const bool sameSelected = selectedFile == it.first;
 				selectedFile = it.first;
-				if(clicks > 1)
+				if(sameSelected && clicks > 1)
 					KeyDown('l', 0, Command(), true);
 				break;
 			}
 	}
 	else
 		return false;
-	
+
 	if(!selectedFile.empty())
 		loadedInfo.Load(Files::Saves() + selectedFile);
-	
+
 	return true;
 }
 
@@ -367,7 +376,7 @@ bool LoadPanel::Hover(int x, int y)
 		sideHasFocus = true;
 	else if(x >= -110 && x < 110)
 		sideHasFocus = false;
-	
+
 	hasHover = true;
 	hoverPoint = Point(x, y);
 	// Tooltips should not pop up unless the mouse stays in one place for the
@@ -375,7 +384,7 @@ bool LoadPanel::Hover(int x, int y)
 	// list, tooltips will appear after one second.
 	if(hoverCount < HOVER_TIME)
 		hoverCount = 0;
-	
+
 	return true;
 }
 
@@ -403,28 +412,32 @@ bool LoadPanel::Scroll(double dx, double dy)
 void LoadPanel::UpdateLists()
 {
 	files.clear();
-	
+
 	vector<string> fileList = Files::List(Files::Saves());
 	for(const string &path : fileList)
 	{
+		// Skip any files that aren't text files.
+		if(path.compare(path.length() - 4, 4, ".txt"))
+			continue;
+
 		string fileName = Files::Name(path);
 		// The file name is either "Pilot Name.txt" or "Pilot Name~SnapshotTitle.txt".
 		size_t pos = fileName.find('~');
 		if(pos == string::npos)
 			pos = fileName.size() - 4;
-		
+
 		string pilotName = fileName.substr(0, pos);
 		files[pilotName].emplace_back(fileName, Files::Timestamp(path));
 	}
-	
+
 	for(auto &it : files)
 		sort(it.second.begin(), it.second.end(),
 			[](const pair<string, time_t> &a, const pair<string, time_t> &b) -> bool
 			{
-				return a.second > b.second;
+				return a.second > b.second || (a.second == b.second && a.first < b.first);
 			}
 		);
-	
+
 	if(!files.empty())
 	{
 		if(selectedPilot.empty())
@@ -449,11 +462,11 @@ void LoadPanel::SnapshotCallback(const string &name)
 	auto it = files.find(selectedPilot);
 	if(it == files.end() || it->second.empty() || it->second.front().first.size() < 4)
 		return;
-	
+
 	string from = Files::Saves() + it->second.front().first;
 	string suffix = name.empty() ? FileDate(from) : name;
 	string extension = "~" + suffix + ".txt";
-	
+
 	// If a file with this name already exists, make sure the player
 	// actually wants to overwrite it.
 	string to = from.substr(0, from.size() - 4) + extension;
@@ -493,11 +506,10 @@ void LoadPanel::LoadCallback()
 	// its background thread is no longer running.
 	gamePanels.Reset();
 	gamePanels.CanSave(true);
-	
+
 	player.Load(loadedInfo.Path());
-	
-	GetUI()->Pop(this);
-	GetUI()->Pop(GetUI()->Root().get());
+
+	GetUI()->PopThrough(GetUI()->Root().get());
 	gamePanels.Push(new MainPanel(player));
 	// It takes one step to figure out the planet panel should be created, and
 	// another step to actually place it. So, take two steps to avoid a flicker.
@@ -507,7 +519,7 @@ void LoadPanel::LoadCallback()
 
 
 
-void LoadPanel::DeletePilot()
+void LoadPanel::DeletePilot(const string &)
 {
 	loadedInfo.Clear();
 	if(selectedPilot == player.Identifier())
@@ -515,7 +527,7 @@ void LoadPanel::DeletePilot()
 	auto it = files.find(selectedPilot);
 	if(it == files.end())
 		return;
-	
+
 	bool failed = false;
 	for(const auto &fit : it->second)
 	{
@@ -525,7 +537,7 @@ void LoadPanel::DeletePilot()
 	}
 	if(failed)
 		GetUI()->Push(new Dialog("Deleting pilot files failed."));
-	
+
 	sideHasFocus = true;
 	selectedPilot.clear();
 	selectedFile.clear();
@@ -542,11 +554,11 @@ void LoadPanel::DeleteSave()
 	Files::Delete(path);
 	if(Files::Exists(path))
 		GetUI()->Push(new Dialog("Deleting snapshot file failed."));
-	
+
 	sideHasFocus = true;
 	selectedPilot.clear();
 	UpdateLists();
-	
+
 	auto it = files.find(pilot);
 	if(it != files.end() && !it->second.empty())
 	{
