@@ -7,22 +7,26 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "OutfitterPanel.h"
 
 #include "text/alignment.hpp"
+#include "comparators/BySeriesAndIndex.h"
 #include "Color.h"
 #include "Dialog.h"
 #include "text/DisplayText.h"
-#include "DistanceMap.h"
 #include "text/Font.h"
 #include "text/FontSet.h"
 #include "text/Format.h"
 #include "GameData.h"
 #include "Hardpoint.h"
 #include "text/layout.hpp"
+#include "Mission.h"
 #include "Outfit.h"
 #include "Planet.h"
 #include "PlayerInfo.h"
@@ -42,9 +46,31 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 using namespace std;
 
 namespace {
-	string Tons(int tons)
+	// Determine the refillable ammunition a particular ship consumes or stores.
+	set<const Outfit *> GetRefillableAmmunition(const Ship &ship) noexcept
 	{
-		return to_string(tons) + (tons == 1 ? " ton" : " tons");
+		auto toRefill = set<const Outfit *>{};
+		auto armed = set<const Outfit *>{};
+		for(auto &&it : ship.Weapons())
+			if(it.GetOutfit())
+			{
+				const Outfit *weapon = it.GetOutfit();
+				armed.emplace(weapon);
+				if(weapon->Ammo() && weapon->AmmoUsage() > 0)
+					toRefill.emplace(weapon->Ammo());
+			}
+
+		// Carriers may be configured to supply ammunition for carried ships found
+		// within the fleet. Since a particular ammunition outfit is not bound to
+		// any particular weapon (i.e. one weapon may consume it, while another may
+		// only require it be installed), we always want to restock these outfits.
+		for(auto &&it : ship.Outfits())
+		{
+			const Outfit *outfit = it.first;
+			if(outfit->Ammo() && !outfit->IsWeapon() && !armed.count(outfit))
+				toRefill.emplace(outfit->Ammo());
+		}
+		return toRefill;
 	}
 }
 
@@ -54,35 +80,28 @@ OutfitterPanel::OutfitterPanel(PlayerInfo &player)
 	: ShopPanel(player, true)
 {
 	for(const pair<const string, Outfit> &it : GameData::Outfits())
-		catalog[it.second.Category()].insert(it.first);
-	
-	// Add owned licenses
-	const string PREFIX = "license: ";
-	for(auto &it : player.Conditions())
-		if(it.first.compare(0, PREFIX.length(), PREFIX) == 0 && it.second > 0)
-		{
-			const string name = it.first.substr(PREFIX.length()) + " License";
-			const Outfit *outfit = GameData::Outfits().Get(name);
-			if(outfit)
-				catalog[outfit->Category()].insert(name);
-		}
-	
+		catalog[it.second.Category()].push_back(it.first);
+
+	for(pair<const string, vector<string>> &it : catalog)
+		sort(it.second.begin(), it.second.end(), BySeriesAndIndex<Outfit>());
+
 	if(player.GetPlanet())
 		outfitter = player.GetPlanet()->Outfitter();
 }
 
 
-	
+
 void OutfitterPanel::Step()
 {
 	CheckRefill();
 	ShopPanel::Step();
-	if(GetUI()->IsTop(this) && !checkedHelp) {
-		if(!DoHelp("outfitter") && !DoHelp("outfitter 2") && !DoHelp("outfitter 3")) {
-			// All help messages have now been displayed.
+	ShopPanel::CheckForMissions(Mission::OUTFITTER);
+	if(GetUI()->IsTop(this) && !checkedHelp)
+		// Use short-circuiting to only display one of them at a time.
+		// (The first valid condition encountered will make us skip the others.)
+		if(DoHelp("outfitter") || DoHelp("cargo management") || DoHelp("uninstalling and storage") || true)
+			// Either a help message was freshly displayed, or all of them have already been seen.
 			checkedHelp = true;
-		}
-	}
 }
 
 
@@ -94,11 +113,18 @@ int OutfitterPanel::TileSize() const
 
 
 
+int OutfitterPanel::VisibilityCheckboxesSize() const
+{
+	return 30;
+}
+
+
+
 int OutfitterPanel::DrawPlayerShipInfo(const Point &point)
 {
-	shipInfo.Update(*playerShip, player.FleetDepreciation(), day);
+	shipInfo.Update(*playerShip, player, collapsed.count("description"));
 	shipInfo.DrawAttributes(point);
-	
+
 	return shipInfo.AttributesHeight();
 }
 
@@ -107,22 +133,22 @@ int OutfitterPanel::DrawPlayerShipInfo(const Point &point)
 bool OutfitterPanel::HasItem(const string &name) const
 {
 	const Outfit *outfit = GameData::Outfits().Get(name);
-	if((outfitter.Has(outfit) || player.Stock(outfit) > 0) && showForSale)
+	if(showForSale && (outfitter.Has(outfit) || player.Stock(outfit) > 0))
 		return true;
-	
-	if(player.Cargo().Get(outfit) && (!playerShip || showForSale))
+
+	if(showCargo && player.Cargo().Get(outfit))
 		return true;
-	
-	if(player.Storage() && player.Storage()->Get(outfit))
+
+	if(showStorage && player.Storage() && player.Storage()->Get(outfit))
 		return true;
-	
+
 	for(const Ship *ship : playerShips)
 		if(ship->OutfitCount(outfit))
 			return true;
-	
+
 	if(showForSale && HasLicense(name))
 		return true;
-	
+
 	return false;
 }
 
@@ -134,15 +160,15 @@ void OutfitterPanel::DrawItem(const string &name, const Point &point, int scroll
 	zones.emplace_back(point, Point(OUTFIT_SIZE, OUTFIT_SIZE), outfit, scrollY);
 	if(point.Y() + OUTFIT_SIZE / 2 < Screen::Top() || point.Y() - OUTFIT_SIZE / 2 > Screen::Bottom())
 		return;
-	
+
 	bool isSelected = (outfit == selectedOutfit);
 	bool isOwned = playerShip && playerShip->OutfitCount(outfit);
 	DrawOutfit(*outfit, point, isSelected, isOwned);
-	
+
 	// Check if this outfit is a "license".
 	bool isLicense = IsLicense(name);
 	int mapSize = outfit->Get("map");
-	
+
 	const Font &font = FontSet::Get(14);
 	const Color &bright = *GameData::Colors().Get("bright");
 	if(playerShip || isLicense || mapSize)
@@ -150,9 +176,9 @@ void OutfitterPanel::DrawItem(const string &name, const Point &point, int scroll
 		int minCount = numeric_limits<int>::max();
 		int maxCount = 0;
 		if(isLicense)
-			minCount = maxCount = player.GetCondition(LicenseName(name));
+			minCount = maxCount = player.HasLicense(LicenseRoot(name));
 		else if(mapSize)
-			minCount = maxCount = HasMapped(mapSize);
+			minCount = maxCount = player.HasMapped(mapSize);
 		else
 		{
 			for(const Ship *ship : playerShips)
@@ -162,13 +188,13 @@ void OutfitterPanel::DrawItem(const string &name, const Point &point, int scroll
 				maxCount = max(maxCount, count);
 			}
 		}
-		
+
 		if(maxCount)
 		{
 			string label = "installed: " + to_string(minCount);
 			if(maxCount > minCount)
 				label += " - " + to_string(maxCount);
-			
+
 			Point labelPos = point + Point(-OUTFIT_SIZE / 2 + 20, OUTFIT_SIZE / 2 - 38);
 			font.Draw(label, labelPos, bright);
 		}
@@ -180,7 +206,7 @@ void OutfitterPanel::DrawItem(const string &name, const Point &point, int scroll
 		stock = max(0, player.Stock(outfit));
 	int cargo = player.Cargo().Get(outfit);
 	int storage = player.Storage() ? player.Storage()->Get(outfit) : 0;
-	
+
 	string message;
 	if(cargo && storage && stock)
 		message = "cargo+stored: " + to_string(cargo + storage) + ", in stock: " + to_string(stock);
@@ -230,28 +256,28 @@ int OutfitterPanel::DrawDetails(const Point &center)
 	const Color &bright = *GameData::Colors().Get("bright");
 	const Color &dim = *GameData::Colors().Get("medium");
 	const Sprite *collapsedArrow = SpriteSet::Get("ui/collapsed");
-	
+
 	int heightOffset = 20;
-	
+
 	if(selectedOutfit)
 	{
-		outfitInfo.Update(*selectedOutfit, player, CanSell());
-		selectedItem = selectedOutfit->Name();
-		
+		outfitInfo.Update(*selectedOutfit, player, CanSell(), collapsed.count("description"));
+		selectedItem = selectedOutfit->DisplayName();
+
 		const Sprite *thumbnail = selectedOutfit->Thumbnail();
 		const Sprite *background = SpriteSet::Get("ui/outfitter selected");
-		
+
 		float tileSize = thumbnail
 			? max(thumbnail->Height(), static_cast<float>(TileSize()))
 			: static_cast<float>(TileSize());
-		
-		Point thumbnailCenter(center.X(), center.Y() + 20 + tileSize / 2);
-		
+
+		Point thumbnailCenter(center.X(), center.Y() + 20 + static_cast<int>(tileSize / 2));
+
 		Point startPoint(center.X() - INFOBAR_WIDTH / 2 + 20, center.Y() + 20 + tileSize);
-		
+
 		double descriptionOffset = 35.;
 		Point descCenter(Screen::Right() - SIDE_WIDTH + INFOBAR_WIDTH / 2, startPoint.Y() + 20.);
-		
+
 		// Maintenance note: This can be replaced with collapsed.contains() in C++20
 		if(!collapsed.count("description"))
 		{
@@ -264,11 +290,12 @@ int OutfitterPanel::DrawDetails(const Point &center)
 			font.Draw(label, startPoint + Point(35., 12.), dim);
 			SpriteShader::Draw(collapsedArrow, startPoint + Point(20., 20.));
 		}
-		
+
 		// Calculate the new ClickZone for the description.
 		Point descDimensions(INFOBAR_WIDTH, descriptionOffset + 10.);
-		ClickZone<std::string> collapseDescription = ClickZone<std::string>(descCenter, descDimensions, std::string("description"));
-		
+		ClickZone<std::string> collapseDescription = ClickZone<std::string>(descCenter,
+			descDimensions, std::string("description"));
+
 		// Find the old zone, and replace it with the new zone.
 		for(auto it = categoryZones.begin(); it != categoryZones.end(); ++it)
 		{
@@ -279,119 +306,220 @@ int OutfitterPanel::DrawDetails(const Point &center)
 			}
 		}
 		categoryZones.emplace_back(collapseDescription);
-		
-		Point attrPoint(startPoint.X(), startPoint.Y() + descriptionOffset);
-		Point reqsPoint(startPoint.X(), attrPoint.Y() + outfitInfo.AttributesHeight());
-		
+
+		Point reqsPoint(startPoint.X(), startPoint.Y() + descriptionOffset);
+		Point attrPoint(startPoint.X(), reqsPoint.Y() + outfitInfo.RequirementsHeight());
+
 		SpriteShader::Draw(background, thumbnailCenter);
 		if(thumbnail)
 			SpriteShader::Draw(thumbnail, thumbnailCenter);
-		
-		outfitInfo.DrawAttributes(attrPoint);
+
 		outfitInfo.DrawRequirements(reqsPoint);
-		
-		heightOffset = reqsPoint.Y() + outfitInfo.RequirementsHeight();
+		outfitInfo.DrawAttributes(attrPoint);
+
+		heightOffset = attrPoint.Y() + outfitInfo.AttributesHeight();
 	}
-	
+
 	// Draw this string representing the selected item (if any), centered in the details side panel
 	Point selectedPoint(center.X() - .5 * INFOBAR_WIDTH, center.Y());
 	font.Draw({selectedItem, {INFOBAR_WIDTH - 20, Alignment::CENTER, Truncate::MIDDLE}},
 		selectedPoint, bright);
-	
+
 	return heightOffset;
 }
 
 
 
-bool OutfitterPanel::CanBuy(bool checkAlreadyOwned) const
+ShopPanel::BuyResult OutfitterPanel::CanBuy(bool onlyOwned) const
 {
 	if(!planet || !selectedOutfit)
 		return false;
-	
-	bool isAlreadyOwned = checkAlreadyOwned && IsAlreadyOwned();
-	if(!(outfitter.Has(selectedOutfit) || player.Stock(selectedOutfit) > 0 || isAlreadyOwned))
-		return false;
-	
+
+	// Check special unique outfits, if you already have them.
 	int mapSize = selectedOutfit->Get("map");
-	if(mapSize > 0 && HasMapped(mapSize))
-		return false;
-	
-	// Determine what you will have to pay to buy this outfit.
-	int64_t cost = player.StockDepreciation().Value(selectedOutfit, day);
+	if(mapSize > 0 && player.HasMapped(mapSize))
+		return "You have already mapped all the systems shown by this map, "
+			"so there is no reason to buy another.";
+
+	if(HasLicense(selectedOutfit->TrueName()))
+		return "You already have one of these licenses, "
+			"so there is no reason to buy another.";
+
 	// Check that the player has any necessary licenses.
-	int64_t licenseCost = LicenseCost(selectedOutfit);
+	int64_t licenseCost = LicenseCost(selectedOutfit, onlyOwned);
 	if(licenseCost < 0)
-		return false;
-	cost += licenseCost;
-	// If you have this in your cargo hold or in planetary storage, installing it is free.
-	if(cost > player.Accounts().Credits() && !isAlreadyOwned)
-		return false;
-	
-	if(HasLicense(selectedOutfit->Name()))
-		return false;
-	
+		return "You cannot buy this outfit, because it requires a license that you don't have.";
+
+	// Check if the outfit is available to get at all.
+	bool isInCargo = player.Cargo().Get(selectedOutfit);
+	bool isInStorage = player.Storage() && player.Storage()->Get(selectedOutfit);
+	bool isInStore = outfitter.Has(selectedOutfit) || player.Stock(selectedOutfit) > 0;
+	if(isInStorage && (onlyOwned || isInStore || playerShip))
+	{
+		// In storage, the outfit is certainly available to get,
+		// except for this one case: 'b' does not move storage to cargo.
+	}
+	else if(isInCargo && playerShip)
+	{
+		// Installing to a ship will work from cargo.
+	}
+	else if(onlyOwned)
+	{
+		// Not using the store, there's nowhere to get this outfit.
+		if(isInStore)
+		{
+			// Player hit 'i' or 'c' when they should've hit 'b'.
+			if(playerShip)
+				return "You'll need to buy this outfit to install it (using \"b\").";
+			else
+				return "You'll need to buy this outfit to put it in your cargo hold (using \"b\").";
+		}
+		else
+		{
+			// Player hit 'i' or 'c' to install, with no outfit to use.
+			if(playerShip)
+				return "You do not have any of these outfits available to install.";
+			else
+				return "You do not have any of these outfits in storage to move to your cargo hold.";
+		}
+	}
+	else if(!isInStore)
+	{
+		// The store doesn't have it.
+		return "You cannot buy this outfit here. "
+			"It is being shown in the list because you have one, "
+			"but this " + planet->Noun() + " does not sell them.";
+	}
+
+	// Check if you need to pay, and can't afford it.
+	if(!onlyOwned)
+	{
+		// Determine what you will have to pay to buy this outfit.
+		int64_t cost = player.StockDepreciation().Value(selectedOutfit, day);
+		int64_t credits = player.Accounts().Credits();
+
+		if(cost > credits)
+			return "You cannot buy this outfit, because it costs "
+				+ Format::CreditString(cost) + ", and you only have "
+				+ Format::Credits(credits) + ".";
+
+		// Add the cost to buy the required license.
+		if(cost + licenseCost > credits)
+			return "You don't have enough money to buy this outfit, because it will cost you an extra "
+				+ Format::CreditString(licenseCost) + " to buy the necessary licenses.";
+	}
+
+	// Check if the outfit will fit
 	if(!playerShip)
 	{
+		// Buying into cargo, so check cargo space vs mass.
 		double mass = selectedOutfit->Mass();
-		return (!mass || player.Cargo().Free() >= mass);
-	}
-	
-	for(const Ship *ship : playerShips)
-		if(ShipCanBuy(ship, selectedOutfit))
+		double freeCargo = player.Cargo().FreePrecise();
+		if(!mass || freeCargo >= mass)
 			return true;
-	
-	return false;
+
+		return "You cannot " + string(onlyOwned ? "load" : "buy") + " this outfit, because it takes up "
+			+ Format::CargoString(mass, "mass") + " and your fleet has "
+			+ Format::CargoString(freeCargo, "cargo space") + " free.";
+	}
+	else
+	{
+		// Find if any ship can install the outfit.
+		for(const Ship *ship : playerShips)
+			if(ShipCanBuy(ship, selectedOutfit))
+				return true;
+
+		// If no selected ship can install the outfit,
+		// report error based on playerShip.
+		double outfitNeeded = -selectedOutfit->Get("outfit space");
+		double outfitSpace = playerShip->Attributes().Get("outfit space");
+		if(outfitNeeded > outfitSpace)
+			return "You cannot install this outfit, because it takes up "
+				+ Format::CargoString(outfitNeeded, "outfit space") + ", and this ship has "
+				+ Format::MassString(outfitSpace) + " free.";
+
+		double weaponNeeded = -selectedOutfit->Get("weapon capacity");
+		double weaponSpace = playerShip->Attributes().Get("weapon capacity");
+		if(weaponNeeded > weaponSpace)
+			return "Only part of your ship's outfit capacity is usable for weapons. "
+				"You cannot install this outfit, because it takes up "
+				+ Format::CargoString(weaponNeeded, "weapon space") + ", and this ship has "
+				+ Format::MassString(weaponSpace) + " free.";
+
+		double engineNeeded = -selectedOutfit->Get("engine capacity");
+		double engineSpace = playerShip->Attributes().Get("engine capacity");
+		if(engineNeeded > engineSpace)
+			return "Only part of your ship's outfit capacity is usable for engines. "
+				"You cannot install this outfit, because it takes up "
+				+ Format::CargoString(engineNeeded, "engine space") + ", and this ship has "
+				+ Format::MassString(engineSpace) + " free.";
+
+		if(selectedOutfit->Category() == "Ammunition")
+			return !playerShip->OutfitCount(selectedOutfit) ?
+				"This outfit is ammunition for a weapon. "
+				"You cannot install it without first installing the appropriate weapon."
+				: "You already have the maximum amount of ammunition for this weapon. "
+				"If you want to install more ammunition, you must first install another of these weapons.";
+
+		int mountsNeeded = -selectedOutfit->Get("turret mounts");
+		int mountsFree = playerShip->Attributes().Get("turret mounts");
+		if(mountsNeeded && !mountsFree)
+			return "This weapon is designed to be installed on a turret mount, "
+				"but your ship does not have any unused turret mounts available.";
+
+		int gunsNeeded = -selectedOutfit->Get("gun ports");
+		int gunsFree = playerShip->Attributes().Get("gun ports");
+		if(gunsNeeded && !gunsFree)
+			return "This weapon is designed to be installed in a gun port, "
+				"but your ship does not have any unused gun ports available.";
+
+		if(selectedOutfit->Get("installable") < 0.)
+			return "This item is not an outfit that can be installed in a ship.";
+
+		// For unhandled outfit requirements, show a catch-all error message.
+		return "You cannot install this outfit in your ship, "
+			"because it would reduce one of your ship's attributes to a negative amount. "
+			"For example, it may use up more cargo space than you have left.";
+	}
 }
 
 
 
-void OutfitterPanel::Buy(bool alreadyOwned)
+void OutfitterPanel::Buy(bool onlyOwned)
 {
-	int64_t licenseCost = LicenseCost(selectedOutfit);
+	int64_t licenseCost = LicenseCost(selectedOutfit, onlyOwned);
 	if(licenseCost)
 	{
 		player.Accounts().AddCredits(-licenseCost);
 		for(const string &licenseName : selectedOutfit->Licenses())
-			if(!player.GetCondition("license: " + licenseName))
-				player.Conditions()["license: " + licenseName] = true;
+			if(!player.HasLicense(licenseName))
+				player.AddLicense(licenseName);
 	}
-	
+
+	// Special case: maps.
+	int mapSize = selectedOutfit->Get("map");
+	if(mapSize)
+	{
+		player.Map(mapSize);
+		player.Accounts().AddCredits(-selectedOutfit->Cost());
+		return;
+	}
+
+	// Special case: licenses.
+	if(IsLicense(selectedOutfit->TrueName()))
+	{
+		player.AddLicense(LicenseRoot(selectedOutfit->TrueName()));
+		player.Accounts().AddCredits(-selectedOutfit->Cost());
+		return;
+	}
+
 	int modifier = Modifier();
-	for(int i = 0; i < modifier && CanBuy(alreadyOwned); ++i)
+	for(int i = 0; i < modifier && CanBuy(onlyOwned); ++i)
 	{
-		// Special case: maps.
-		int mapSize = selectedOutfit->Get("map");
-		if(mapSize > 0)
-		{
-			if(!HasMapped(mapSize))
-			{
-				DistanceMap distance(player.GetSystem(), mapSize);
-				for(const System *system : distance.Systems())
-					if(!player.HasVisited(*system))
-						player.Visit(*system);
-				int64_t price = player.StockDepreciation().Value(selectedOutfit, day);
-				player.Accounts().AddCredits(-price);
-			}
-			return;
-		}
-		
-		// Special case: licenses.
-		if(IsLicense(selectedOutfit->Name()))
-		{
-			auto &entry = player.Conditions()[LicenseName(selectedOutfit->Name())];
-			if(entry <= 0)
-			{
-				entry = true;
-				int64_t price = player.StockDepreciation().Value(selectedOutfit, day);
-				player.Accounts().AddCredits(-price);
-			}
-			return;
-		}
-		
 		// Buying into cargo, either from storage or from stock/supply.
 		if(!playerShip)
 		{
-			if(alreadyOwned)
+			if(onlyOwned)
 			{
 				if(!player.Storage() || !player.Storage()->Get(selectedOutfit))
 					continue;
@@ -400,7 +528,7 @@ void OutfitterPanel::Buy(bool alreadyOwned)
 			}
 			else
 			{
-				// Check if the outfit is for sale or in stock so that we can actualy buy it.
+				// Check if the outfit is for sale or in stock so that we can actually buy it.
 				if(!outfitter.Has(selectedOutfit) && player.Stock(selectedOutfit) <= 0)
 					continue;
 				player.Cargo().Add(selectedOutfit);
@@ -410,20 +538,20 @@ void OutfitterPanel::Buy(bool alreadyOwned)
 				continue;
 			}
 		}
-		
+
 		// Find the ships with the fewest number of these outfits.
 		const vector<Ship *> shipsToOutfit = GetShipsToOutfit(true);
-		
+
 		for(Ship *ship : shipsToOutfit)
 		{
-			if(!CanBuy(alreadyOwned))
+			if(!CanBuy(onlyOwned))
 				return;
-		
+
 			if(player.Cargo().Get(selectedOutfit))
 				player.Cargo().Remove(selectedOutfit);
 			else if(player.Storage() && player.Storage()->Get(selectedOutfit))
 				player.Storage()->Remove(selectedOutfit);
-			else if(alreadyOwned || !(player.Stock(selectedOutfit) > 0 || outfitter.Has(selectedOutfit)))
+			else if(onlyOwned || !(player.Stock(selectedOutfit) > 0 || outfitter.Has(selectedOutfit)))
 				break;
 			else
 			{
@@ -442,157 +570,21 @@ void OutfitterPanel::Buy(bool alreadyOwned)
 
 
 
-void OutfitterPanel::FailBuy() const
-{
-	if(!selectedOutfit)
-		return;
-	
-	int64_t cost = player.StockDepreciation().Value(selectedOutfit, day);
-	int64_t credits = player.Accounts().Credits();
-	bool isInCargo = player.Cargo().Get(selectedOutfit);
-	bool isInStorage = player.Storage() && player.Storage()->Get(selectedOutfit);
-	if(!isInCargo && !isInStorage && cost > credits)
-	{
-		GetUI()->Push(new Dialog("You cannot buy this outfit, because it costs "
-			+ Format::Credits(cost) + " credits, and you only have "
-			+ Format::Credits(credits) + "."));
-		return;
-	}
-	// Check that the player has any necessary licenses.
-	int64_t licenseCost = LicenseCost(selectedOutfit);
-	if(licenseCost < 0)
-	{
-		GetUI()->Push(new Dialog(
-			"You cannot buy this outfit, because it requires a license that you don't have."));
-		return;
-	}
-	if(!isInCargo && !isInStorage && cost + licenseCost > credits)
-	{
-		GetUI()->Push(new Dialog(
-			"You don't have enough money to buy this outfit, because it will cost you an extra "
-			+ Format::Credits(licenseCost) + " credits to buy the necessary licenses."));
-		return;
-	}
-	
-	if(!(outfitter.Has(selectedOutfit) || player.Stock(selectedOutfit) > 0 || isInCargo || isInStorage))
-	{
-		GetUI()->Push(new Dialog("You cannot buy this outfit here. "
-			"It is being shown in the list because you have one installed in your ship, "
-			"but this " + planet->Noun() + " does not sell them."));
-		return;
-	}
-	
-	if(selectedOutfit->Get("map"))
-	{
-		GetUI()->Push(new Dialog("You have already mapped all the systems shown by this map, "
-			"so there is no reason to buy another."));
-		return;
-	}
-	
-	if(HasLicense(selectedOutfit->Name()))
-	{
-		GetUI()->Push(new Dialog("You already have one of these licenses, "
-			"so there is no reason to buy another."));
-		return;
-	}
-	
-	if(!playerShip)
-		return;
-	
-	double outfitNeeded = -selectedOutfit->Get("outfit space");
-	double outfitSpace = playerShip->Attributes().Get("outfit space");
-	if(outfitNeeded > outfitSpace)
-	{
-		string need =  to_string(outfitNeeded) + (outfitNeeded != 1. ? "tons" : "ton");
-		GetUI()->Push(new Dialog("You cannot install this outfit, because it takes up "
-			+ Tons(outfitNeeded) + " of outfit space, and this ship has "
-			+ Tons(outfitSpace) + " free."));
-		return;
-	}
-	
-	double weaponNeeded = -selectedOutfit->Get("weapon capacity");
-	double weaponSpace = playerShip->Attributes().Get("weapon capacity");
-	if(weaponNeeded > weaponSpace)
-	{
-		GetUI()->Push(new Dialog("Only part of your ship's outfit capacity is usable for weapons. "
-			"You cannot install this outfit, because it takes up "
-			+ Tons(weaponNeeded) + " of weapon space, and this ship has "
-			+ Tons(weaponSpace) + " free."));
-		return;
-	}
-	
-	double engineNeeded = -selectedOutfit->Get("engine capacity");
-	double engineSpace = playerShip->Attributes().Get("engine capacity");
-	if(engineNeeded > engineSpace)
-	{
-		GetUI()->Push(new Dialog("Only part of your ship's outfit capacity is usable for engines. "
-			"You cannot install this outfit, because it takes up "
-			+ Tons(engineNeeded) + " of engine space, and this ship has "
-			+ Tons(engineSpace) + " free."));
-		return;
-	}
-	
-	if(selectedOutfit->Category() == "Ammunition")
-	{
-		if(!playerShip->OutfitCount(selectedOutfit))
-			GetUI()->Push(new Dialog("This outfit is ammunition for a weapon. "
-				"You cannot install it without first installing the appropriate weapon."));
-		else
-			GetUI()->Push(new Dialog("You already have the maximum amount of ammunition for this weapon. "
-				"If you want to install more ammunition, you must first install another of these weapons."));
-		return;
-	}
-	
-	int mountsNeeded = -selectedOutfit->Get("turret mounts");
-	int mountsFree = playerShip->Attributes().Get("turret mounts");
-	if(mountsNeeded && !mountsFree)
-	{
-		GetUI()->Push(new Dialog("This weapon is designed to be installed on a turret mount, "
-			"but your ship does not have any unused turret mounts available."));
-		return;
-	}
-	
-	int gunsNeeded = -selectedOutfit->Get("gun ports");
-	int gunsFree = playerShip->Attributes().Get("gun ports");
-	if(gunsNeeded && !gunsFree)
-	{
-		GetUI()->Push(new Dialog("This weapon is designed to be installed in a gun port, "
-			"but your ship does not have any unused gun ports available."));
-		return;
-	}
-	
-	if(selectedOutfit->Get("installable") < 0.)
-	{
-		GetUI()->Push(new Dialog("This item is not an outfit that can be installed in a ship."));
-		return;
-	}
-	
-	if(!playerShip->Attributes().CanAdd(*selectedOutfit, 1))
-	{
-		GetUI()->Push(new Dialog("You cannot install this outfit in your ship, "
-			"because it would reduce one of your ship's attributes to a negative amount. "
-			"For example, it may use up more cargo space than you have left."));
-		return;
-	}
-}
-
-
-
 bool OutfitterPanel::CanSell(bool toStorage) const
 {
 	if(!planet || !selectedOutfit)
 		return false;
-	
+
 	if(player.Cargo().Get(selectedOutfit))
 		return true;
-		
+
 	if(!toStorage && player.Storage() && player.Storage()->Get(selectedOutfit))
 		return true;
-	
+
 	for(const Ship *ship : playerShips)
 		if(ShipCanSell(ship, selectedOutfit))
 			return true;
-	
+
 	return false;
 }
 
@@ -604,7 +596,7 @@ void OutfitterPanel::Sell(bool toStorage)
 	// we also request storage to be created if possible.
 	// Will be nullptr if no storage is available.
 	CargoHold *storage = player.Storage(toStorage);
-	
+
 	if(player.Cargo().Get(selectedOutfit))
 	{
 		player.Cargo().Remove(selectedOutfit);
@@ -626,7 +618,7 @@ void OutfitterPanel::Sell(bool toStorage)
 	// Get the ships that have the most of this outfit installed.
 	// If there are no ships that have this outfit, then sell from storage.
 	const vector<Ship *> shipsToOutfit = GetShipsToOutfit();
-	
+
 	if(shipsToOutfit.size() > 0)
 	{
 		for(Ship *ship : shipsToOutfit)
@@ -655,7 +647,7 @@ void OutfitterPanel::Sell(bool toStorage)
 				player.Accounts().AddCredits(price);
 				player.AddStock(selectedOutfit, 1);
 			}
-			
+
 			const Outfit *ammo = selectedOutfit->Ammo();
 			if(ammo && ship->OutfitCount(ammo))
 			{
@@ -664,7 +656,7 @@ void OutfitterPanel::Sell(bool toStorage)
 				for(const pair<const char *, double> &it : ship->Attributes().Attributes())
 					if(it.second < 0.)
 						mustSell = max<int>(mustSell, it.second / ammo->Get(it.first));
-				
+
 				if(mustSell)
 				{
 					ship->AddOutfit(ammo, -mustSell);
@@ -681,7 +673,7 @@ void OutfitterPanel::Sell(bool toStorage)
 		}
 		return;
 	}
-	
+
 	if(!toStorage && storage && storage->Get(selectedOutfit))
 	{
 		storage->Remove(selectedOutfit);
@@ -700,7 +692,7 @@ void OutfitterPanel::FailSell(bool toStorage) const
 		return;
 	else if(selectedOutfit->Get("map"))
 		GetUI()->Push(new Dialog("You cannot " + verb + " maps. Once you buy one, it is yours permanently."));
-	else if(HasLicense(selectedOutfit->Name()))
+	else if(HasLicense(selectedOutfit->TrueName()))
 		GetUI()->Push(new Dialog("You cannot " + verb + " licenses. Once you obtain one, it is yours permanently."));
 	else
 	{
@@ -727,7 +719,7 @@ void OutfitterPanel::FailSell(bool toStorage) const
 									"because that would cause your ship's \"" + it.first +
 									"\" value to be reduced to less than zero. "
 									"To " + verb + " this outfit, you must " + verb + " the " +
-									sit.first->Name() + " outfit first."));
+									sit.first->DisplayName() + " outfit first."));
 								return;
 							}
 						GetUI()->Push(new Dialog("You cannot " + verb + " this outfit, "
@@ -747,12 +739,12 @@ bool OutfitterPanel::ShouldHighlight(const Ship *ship)
 {
 	if(!selectedOutfit)
 		return false;
-	
+
 	if(hoverButton == 'b')
 		return CanBuy() && ShipCanBuy(ship, selectedOutfit);
 	else if(hoverButton == 's')
 		return CanSell() && ShipCanSell(ship, selectedOutfit);
-	
+
 	return false;
 }
 
@@ -762,22 +754,26 @@ void OutfitterPanel::DrawKey()
 {
 	const Sprite *back = SpriteSet::Get("ui/outfitter key");
 	SpriteShader::Draw(back, Screen::BottomLeft() + .5 * Point(back->Width(), -back->Height()));
-	
+
 	const Font &font = FontSet::Get(14);
 	Color color[2] = {*GameData::Colors().Get("medium"), *GameData::Colors().Get("bright")};
 	const Sprite *box[2] = {SpriteSet::Get("ui/unchecked"), SpriteSet::Get("ui/checked")};
-	
-	Point pos = Screen::BottomLeft() + Point(10., -30.);
+
+	Point pos = Screen::BottomLeft() + Point(10., -VisibilityCheckboxesSize() - 20.);
 	Point off = Point(10., -.5 * font.Height());
 	SpriteShader::Draw(box[showForSale], pos);
 	font.Draw("Show outfits for sale", pos + off, color[showForSale]);
 	AddZone(Rectangle(pos + Point(80., 0.), Point(180., 20.)), [this](){ ToggleForSale(); });
-	
-	bool showCargo = !playerShip;
+
 	pos.Y() += 20.;
 	SpriteShader::Draw(box[showCargo], pos);
 	font.Draw("Show outfits in cargo", pos + off, color[showCargo]);
 	AddZone(Rectangle(pos + Point(80., 0.), Point(180., 20.)), [this](){ ToggleCargo(); });
+
+	pos.Y() += 20.;
+	SpriteShader::Draw(box[showStorage], pos);
+	font.Draw("Show outfits in storage", pos + off, color[showStorage]);
+	AddZone(Rectangle(pos + Point(80., 0.), Point(180., 20.)), [this](){ ToggleStorage(); });
 }
 
 
@@ -785,33 +781,52 @@ void OutfitterPanel::DrawKey()
 void OutfitterPanel::ToggleForSale()
 {
 	showForSale = !showForSale;
-	
+
+	if(selectedOutfit && !HasItem(selectedOutfit->TrueName()))
+	{
+		selectedOutfit = nullptr;
+	}
+
 	ShopPanel::ToggleForSale();
 }
 
 
 
+void OutfitterPanel::ToggleStorage()
+{
+	showStorage = !showStorage;
+
+	if(selectedOutfit && !HasItem(selectedOutfit->TrueName()))
+	{
+		selectedOutfit = nullptr;
+	}
+
+	ShopPanel::ToggleStorage();
+}
+
+
+
 void OutfitterPanel::ToggleCargo()
 {
+	showCargo = !showCargo;
+
+	if(selectedOutfit && !HasItem(selectedOutfit->TrueName()))
+	{
+		selectedOutfit = nullptr;
+	}
+
 	if(playerShip)
 	{
-		previousShip = playerShip;
 		playerShip = nullptr;
-		previousShips = playerShips;
 		playerShips.clear();
 	}
-	else if(previousShip)
-	{
-		playerShip = previousShip;
-		playerShips = previousShips;
-	}
 	else
 	{
 		playerShip = player.Flagship();
 		if(playerShip)
 			playerShips.insert(playerShip);
 	}
-	
+
 	ShopPanel::ToggleCargo();
 }
 
@@ -828,7 +843,7 @@ bool OutfitterPanel::ShipCanSell(const Ship *ship, const Outfit *outfit)
 {
 	if(!ship->OutfitCount(outfit))
 		return false;
-	
+
 	// If this outfit requires ammo, check if we could sell it if we sold all
 	// the ammo for it first.
 	const Outfit *ammo = outfit->Ammo();
@@ -838,7 +853,7 @@ bool OutfitterPanel::ShipCanSell(const Ship *ship, const Outfit *outfit)
 		attributes.Add(*ammo, -ship->OutfitCount(ammo));
 		return attributes.CanAdd(*outfit, -1);
 	}
-	
+
 	// Now, check whether this ship can sell this outfit.
 	return ship->Attributes().CanAdd(*outfit, -1);
 }
@@ -852,9 +867,9 @@ void OutfitterPanel::DrawOutfit(const Outfit &outfit, const Point &center, bool
 		isSelected ? "ui/outfitter selected" : "ui/outfitter unselected");
 	SpriteShader::Draw(back, center);
 	SpriteShader::Draw(thumbnail, center);
-	
+
 	// Draw the outfit name.
-	const string &name = outfit.Name();
+	const string &name = outfit.DisplayName();
 	const Font &font = FontSet::Get(14);
 	Point offset(-.5 * OUTFIT_SIZE, -.5 * OUTFIT_SIZE + 10.);
 	font.Draw({name, {OUTFIT_SIZE, Alignment::CENTER, Truncate::MIDDLE}},
@@ -863,18 +878,6 @@ void OutfitterPanel::DrawOutfit(const Outfit &outfit, const Point &center, bool
 
 
 
-bool OutfitterPanel::HasMapped(int mapSize) const
-{
-	DistanceMap distance(player.GetSystem(), mapSize);
-	for(const System *system : distance.Systems())
-		if(!player.HasVisited(*system))
-			return false;
-	
-	return true;
-}
-
-
-
 bool OutfitterPanel::IsLicense(const string &name) const
 {
 	static const string &LICENSE = " License";
@@ -882,7 +885,7 @@ bool OutfitterPanel::IsLicense(const string &name) const
 		return false;
 	if(name.compare(name.length() - LICENSE.length(), LICENSE.length(), LICENSE))
 		return false;
-	
+
 	return true;
 }
 
@@ -890,15 +893,15 @@ bool OutfitterPanel::IsLicense(const string &name) const
 
 bool OutfitterPanel::HasLicense(const string &name) const
 {
-	return (IsLicense(name) && player.GetCondition(LicenseName(name)) > 0);
+	return (IsLicense(name) && player.HasLicense(LicenseRoot(name)));
 }
 
 
 
-string OutfitterPanel::LicenseName(const string &name) const
+string OutfitterPanel::LicenseRoot(const string &name) const
 {
 	static const string &LICENSE = " License";
-	return "license: " + name.substr(0, name.length() - LICENSE.length());
+	return name.substr(0, name.length() - LICENSE.length());
 }
 
 
@@ -908,20 +911,17 @@ void OutfitterPanel::CheckRefill()
 	if(checkedRefill)
 		return;
 	checkedRefill = true;
-	
+
 	int count = 0;
 	map<const Outfit *, int> needed;
 	for(const shared_ptr<Ship> &ship : player.Ships())
 	{
+		// Skip ships in other systems and those that were unable to land in-system.
 		if(ship->GetSystem() != player.GetSystem() || ship->IsDisabled())
 			continue;
-		
+
 		++count;
-		set<const Outfit *> toRefill;
-		for(const Hardpoint &it : ship->Weapons())
-			if(it.GetOutfit() && it.GetOutfit()->Ammo())
-				toRefill.insert(it.GetOutfit()->Ammo());
-		
+		auto toRefill = GetRefillableAmmunition(*ship);
 		for(const Outfit *outfit : toRefill)
 		{
 			int amount = ship->Attributes().CanAdd(*outfit, numeric_limits<int>::max());
@@ -929,7 +929,7 @@ void OutfitterPanel::CheckRefill()
 				needed[outfit] += amount;
 		}
 	}
-	
+
 	int64_t cost = 0;
 	for(auto &it : needed)
 	{
@@ -944,7 +944,7 @@ void OutfitterPanel::CheckRefill()
 		string message = "Do you want to reload all the ammunition for your ship";
 		message += (count == 1) ? "?" : "s?";
 		if(cost)
-			message += " It will cost " + Format::Credits(cost) + " credits.";
+			message += " It will cost " + Format::CreditString(cost) + ".";
 		GetUI()->Push(new Dialog(this, &OutfitterPanel::Refill, message));
 	}
 }
@@ -955,14 +955,11 @@ void OutfitterPanel::Refill()
 {
 	for(const shared_ptr<Ship> &ship : player.Ships())
 	{
+		// Skip ships in other systems and those that were unable to land in-system.
 		if(ship->GetSystem() != player.GetSystem() || ship->IsDisabled())
 			continue;
-		
-		set<const Outfit *> toRefill;
-		for(const Hardpoint &it : ship->Weapons())
-			if(it.GetOutfit() && it.GetOutfit()->Ammo())
-				toRefill.insert(it.GetOutfit()->Ammo());
-		
+
+		auto toRefill = GetRefillableAmmunition(*ship);
 		for(const Outfit *outfit : toRefill)
 		{
 			int neededAmmo = ship->Attributes().CanAdd(*outfit, numeric_limits<int>::max());
@@ -999,7 +996,7 @@ const vector<Ship *> OutfitterPanel::GetShipsToOutfit(bool isBuy) const
 		if((isBuy && !ShipCanBuy(ship, selectedOutfit))
 				|| (!isBuy && !ShipCanSell(ship, selectedOutfit)))
 			continue;
-		
+
 		int count = ship->OutfitCount(selectedOutfit);
 		if(compareMod * count < compareMod * compareValue)
 		{
@@ -1009,6 +1006,6 @@ const vector<Ship *> OutfitterPanel::GetShipsToOutfit(bool isBuy) const
 		if(count == compareValue)
 			shipsToOutfit.push_back(ship);
 	}
-	
+
 	return shipsToOutfit;
 }
