@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Panel.h"
@@ -16,6 +19,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Command.h"
 #include "Dialog.h"
 #include "FillShader.h"
+#include "text/Format.h"
 #include "GameData.h"
 #include "Point.h"
 #include "Preferences.h"
@@ -29,20 +33,14 @@ using namespace std;
 // Move the state of this panel forward one game step.
 void Panel::Step()
 {
-}
-
-
-
-// Draw this panel.
-void Panel::Draw()
-{
+	// It is ok for panels to be stateless.
 }
 
 
 
 // Return true if this is a full-screen panel, so there is no point in
 // drawing any of the panels under it.
-bool Panel::IsFullScreen()
+bool Panel::IsFullScreen() const noexcept
 {
 	return isFullScreen;
 }
@@ -51,7 +49,7 @@ bool Panel::IsFullScreen()
 
 // Return true if, when this panel is on the stack, no events should be
 // passed to any panel under it. By default, all panels do this.
-bool Panel::TrapAllEvents()
+bool Panel::TrapAllEvents() const noexcept
 {
 	return trapAllEvents;
 }
@@ -59,7 +57,7 @@ bool Panel::TrapAllEvents()
 
 
 // Check if this panel can be "interrupted" to return to the main menu.
-bool Panel::IsInterruptible() const
+bool Panel::IsInterruptible() const noexcept
 {
 	return isInterruptible;
 }
@@ -110,9 +108,16 @@ bool Panel::ZoneClick(const Point &point)
 
 
 
+// Forward the given TestContext to the Engine under MainPanel.
+void Panel::SetTestContext(TestContext &testContext)
+{
+}
+
+
+
 // Panels will by default not allow fast-forward. The ones that do allow
 // it will override this (virtual) function and return true.
-bool Panel::AllowFastForward() const
+bool Panel::AllowsFastForward() const noexcept
 {
 	return false;
 }
@@ -168,7 +173,7 @@ bool Panel::Release(int x, int y)
 }
 
 
-	
+
 void Panel::SetIsFullScreen(bool set)
 {
 	isFullScreen = set;
@@ -189,13 +194,13 @@ void Panel::SetInterruptible(bool set)
 }
 
 
-	
+
 // Dim the background of this panel.
 void Panel::DrawBackdrop() const
 {
 	if(!GetUI()->IsTop(this))
 		return;
-	
+
 	// Darken everything but the dialog.
 	const Color &back = *GameData::Colors().Get("dialog backdrop");
 	FillShader::Fill(Point(), Point(Screen::Width(), Screen::Height()), back);
@@ -203,7 +208,7 @@ void Panel::DrawBackdrop() const
 
 
 
-UI *Panel::GetUI() const
+UI *Panel::GetUI() const noexcept
 {
 	return ui;
 }
@@ -226,7 +231,7 @@ bool Panel::DoKey(SDL_Keycode key, Uint16 mod)
 int Panel::Modifier()
 {
 	SDL_Keymod mod = SDL_GetModState();
-	
+
 	int modifier = 1;
 	if(mod & KMOD_ALT)
 		modifier *= 500;
@@ -234,7 +239,7 @@ int Panel::Modifier()
 		modifier *= 20;
 	if(mod & KMOD_SHIFT)
 		modifier *= 5;
-	
+
 	return modifier;
 }
 
@@ -247,14 +252,14 @@ bool Panel::DoHelp(const string &name) const
 	string preference = "help: " + name;
 	if(Preferences::Has(preference))
 		return false;
-	
+
 	const string &message = GameData::HelpMessage(name);
 	if(message.empty())
 		return false;
-	
+
 	Preferences::Set(preference);
-	ui->Push(new Dialog(message));
-	
+	ui->Push(new Dialog(Format::Capitalize(name) + ":\n\n" + message));
+
 	return true;
 }
 
