@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Rectangle.h"
@@ -177,3 +180,10 @@ bool Rectangle::Contains(const Rectangle &other) const
 {
 	return Contains(other.TopLeft()) && Contains(other.BottomRight());
 }
+
+
+
+bool Rectangle::Overlaps(const Rectangle &other) const
+{
+	return !(other.Left() > Right() || other.Right() < Left() || other.Top() > Bottom() || other.Bottom() < Top());
+}
