@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef RING_SHADER_H_
@@ -23,13 +26,15 @@ class Point;
 class RingShader {
 public:
 	static void Init();
-	
+
 	static void Draw(const Point &pos, float out, float in, const Color &color);
-	static void Draw(const Point &pos, float radius, float width, float fraction, const Color &color, float dash = 0.f, float startAngle = 0.f);
-	
+	static void Draw(const Point &pos, float radius, float width, float fraction,
+		const Color &color, float dash = 0.f, float startAngle = 0.f);
+
 	static void Bind();
 	static void Add(const Point &pos, float out, float in, const Color &color);
-	static void Add(const Point &pos, float radius, float width, float fraction, const Color &color, float dash = 0.f, float startAngle = 0.f);
+	static void Add(const Point &pos, float radius, float width, float fraction,
+		const Color &color, float dash = 0.f, float startAngle = 0.f);
 	static void Unbind();
 };
 
