@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef SHIPYARD_PANEL_H_
@@ -33,8 +36,10 @@ class Ship;
 class ShipyardPanel : public ShopPanel {
 public:
 	explicit ShipyardPanel(PlayerInfo &player);
-	
-	
+
+	virtual void Step() override;
+
+
 protected:
 	virtual int TileSize() const override;
 	virtual int DrawPlayerShipInfo(const Point &point) override;
@@ -43,23 +48,21 @@ protected:
 	virtual int DividerOffset() const override;
 	virtual int DetailWidth() const override;
 	virtual int DrawDetails(const Point &center) override;
-	virtual bool CanBuy() const override;
-	virtual void Buy() override;
-	virtual void FailBuy() const override;
-	virtual bool CanSell() const override;
-	virtual void Sell() override;
-	virtual bool FlightCheck() override;
+	virtual BuyResult CanBuy(bool onlyOwned = false) const override;
+	virtual void Buy(bool onlyOwned = false) override;
+	virtual bool CanSell(bool toStorage = false) const override;
+	virtual void Sell(bool toStorage = false) override;
 	virtual bool CanSellMultiple() const override;
-	
-	
+
+
 private:
 	void BuyShip(const std::string &name);
 	void SellShip();
-	
-	
+
+
 private:
 	int modifier;
-	
+
 	Sale<Ship> shipyard;
 };
 
